@@ -2,6 +2,8 @@
 // for details. All rights reserved. Use of this source code is governed by a
 // BSD-style license that can be found in the LICENSE file.
 
+// @dart = 2.6
+
 // Patch file for dart:core classes.
 import "dart:_internal" hide Symbol, LinkedList, LinkedListEntry;
 import "dart:_internal" as _symbol_dev;
@@ -104,7 +106,7 @@ class Expando<T> {
   Expando([String name])
       : this.name = name,
         _jsWeakMapOrKey = JS('bool', 'typeof WeakMap == "function"')
-            ? JS('=Object|Null', 'new WeakMap()')
+            ? JS('=Object', 'new WeakMap()')
             : _createKey();
 
   @patch
@@ -438,6 +440,12 @@ class List<E> {
     return makeListFixedLength(list);
   }
 
+  @patch
+  factory List.of(Iterable<E> elements, {bool growable: true}) {
+    // TODO(32937): Specialize to benefit from known element type.
+    return List.from(elements, growable: growable);
+  }
+
   @patch
   factory List.unmodifiable(Iterable elements) {
     List result = new List<E>.from(elements, growable: false);
@@ -700,8 +708,8 @@ class _Uri {
   static final bool _isWindowsCached = JS(
       'bool',
       'typeof process != "undefined" && '
-      'Object.prototype.toString.call(process) == "[object process]" && '
-      'process.platform == "win32"');
+          'Object.prototype.toString.call(process) == "[object process]" && '
+          'process.platform == "win32"');
 
   // Matches a String that _uriEncodes to itself regardless of the kind of
   // component.  This corresponds to [_unreservedTable], i.e. characters that
@@ -855,6 +863,9 @@ class _DuplicatedFieldInitializerError extends Error {
 int _max(int a, int b) => a > b ? a : b;
 int _min(int a, int b) => a < b ? a : b;
 
+/// Empty list used as an initializer for local variables in the `_BigIntImpl`.
+final _dummyList = new Uint16List(0);
+
 // Copyright 2009 The Go Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style
 // license that can be found in the LICENSE file.
@@ -1071,9 +1082,7 @@ class _BigIntImpl implements BigInt {
   static _BigIntImpl _tryParse(String source, {int radix}) {
     if (source == "") return null;
 
-    var re = new RegExp(r'^\s*([+-]?)((0x[a-f0-9]+)|(\d+)|([a-z0-9]+))\s*$',
-        caseSensitive: false);
-    var match = re.firstMatch(source);
+    var match = _parseRE.firstMatch(source);
     int signIndex = 1;
     int hexIndex = 3;
     int decimalIndex = 4;
@@ -1115,6 +1124,10 @@ class _BigIntImpl implements BigInt {
         decimalMatch ?? nonDecimalMatch ?? hexMatch, radix, isNegative);
   }
 
+  static RegExp _parseRE = RegExp(
+      r'^\s*([+-]?)((0x[a-f0-9]+)|(\d+)|([a-z0-9]+))\s*$',
+      caseSensitive: false);
+
   /// Finds the amount significant digits in the provided [digits] array.
   static int _normalize(int used, Uint16List digits) {
     while (used > 0 && digits[used - 1] == 0) used--;
@@ -1451,8 +1464,7 @@ class _BigIntImpl implements BigInt {
   ///
   /// Returns 0 if abs(this) == abs(other); a positive number if
   /// abs(this) > abs(other); and a negative number if abs(this) < abs(other).
-  int _absCompare(BigInt bigInt) {
-    _BigIntImpl other = bigInt;
+  int _absCompare(_BigIntImpl other) {
     return _compareDigits(_digits, _used, other._digits, other._used);
   }
 
@@ -1460,8 +1472,7 @@ class _BigIntImpl implements BigInt {
   ///
   /// Returns a negative number if `this` is less than `other`, zero if they are
   /// equal, and a positive number if `this` is greater than `other`.
-  int compareTo(BigInt bigInt) {
-    _BigIntImpl other = bigInt;
+  int compareTo(covariant _BigIntImpl other) {
     if (_isNegative == other._isNegative) {
       var result = _absCompare(other);
       // Use 0 - result to avoid negative zero in JavaScript.
@@ -1603,7 +1614,8 @@ class _BigIntImpl implements BigInt {
     var digits = _digits;
     var otherDigits = other._digits;
     var resultDigits = new Uint16List(resultUsed);
-    var l, m;
+    _BigIntImpl l;
+    int m;
     if (used < otherUsed) {
       l = other;
       m = used;
@@ -1629,7 +1641,8 @@ class _BigIntImpl implements BigInt {
     var digits = _digits;
     var otherDigits = other._digits;
     var resultDigits = new Uint16List(resultUsed);
-    var l, m;
+    _BigIntImpl l;
+    int m;
     if (used < otherUsed) {
       l = other;
       m = used;
@@ -1655,8 +1668,7 @@ class _BigIntImpl implements BigInt {
   ///
   /// Of both operands are negative, the result is negative, otherwise
   /// the result is non-negative.
-  _BigIntImpl operator &(BigInt bigInt) {
-    _BigIntImpl other = bigInt;
+  _BigIntImpl operator &(covariant _BigIntImpl other) {
     if (_isZero || other._isZero) return zero;
     if (_isNegative == other._isNegative) {
       if (_isNegative) {
@@ -1671,7 +1683,7 @@ class _BigIntImpl implements BigInt {
       return _absAndSetSign(other, false);
     }
     // _isNegative != other._isNegative
-    var p, n;
+    _BigIntImpl p, n;
     if (_isNegative) {
       p = other;
       n = this;
@@ -1693,8 +1705,7 @@ class _BigIntImpl implements BigInt {
   ///
   /// If both operands are non-negative, the result is non-negative,
   /// otherwise the result us negative.
-  _BigIntImpl operator |(BigInt bigInt) {
-    _BigIntImpl other = bigInt;
+  _BigIntImpl operator |(covariant _BigIntImpl other) {
     if (_isZero) return other;
     if (other._isZero) return this;
     if (_isNegative == other._isNegative) {
@@ -1710,7 +1721,7 @@ class _BigIntImpl implements BigInt {
       return _absOrSetSign(other, false);
     }
     // _neg != a._neg
-    var p, n;
+    _BigIntImpl p, n;
     if (_isNegative) {
       p = other;
       n = this;
@@ -1733,8 +1744,7 @@ class _BigIntImpl implements BigInt {
   ///
   /// If the operands have the same sign, the result is non-negative,
   /// otherwise the result is negative.
-  _BigIntImpl operator ^(BigInt bigInt) {
-    _BigIntImpl other = bigInt;
+  _BigIntImpl operator ^(covariant _BigIntImpl other) {
     if (_isZero) return other;
     if (other._isZero) return this;
     if (_isNegative == other._isNegative) {
@@ -1747,7 +1757,7 @@ class _BigIntImpl implements BigInt {
       return _absXorSetSign(other, false);
     }
     // _isNegative != a._isNegative
-    var p, n;
+    _BigIntImpl p, n;
     if (_isNegative) {
       p = other;
       n = this;
@@ -1780,8 +1790,7 @@ class _BigIntImpl implements BigInt {
   }
 
   /// Addition operator.
-  _BigIntImpl operator +(BigInt bigInt) {
-    _BigIntImpl other = bigInt;
+  _BigIntImpl operator +(covariant _BigIntImpl other) {
     if (_isZero) return other;
     if (other._isZero) return this;
     var isNegative = _isNegative;
@@ -1799,8 +1808,7 @@ class _BigIntImpl implements BigInt {
   }
 
   /// Subtraction operator.
-  _BigIntImpl operator -(BigInt bigInt) {
-    _BigIntImpl other = bigInt;
+  _BigIntImpl operator -(covariant _BigIntImpl other) {
     if (_isZero) return -other;
     if (other._isZero) return this;
     var isNegative = _isNegative;
@@ -1852,8 +1860,7 @@ class _BigIntImpl implements BigInt {
   }
 
   /// Multiplication operator.
-  _BigIntImpl operator *(BigInt bigInt) {
-    _BigIntImpl other = bigInt;
+  _BigIntImpl operator *(covariant _BigIntImpl other) {
     var used = _used;
     var otherUsed = other._used;
     if (used == 0 || otherUsed == 0) {
@@ -1901,8 +1908,7 @@ class _BigIntImpl implements BigInt {
   }
 
   /// Returns `trunc(this / other)`, with `other != 0`.
-  _BigIntImpl _div(BigInt bigInt) {
-    _BigIntImpl other = bigInt;
+  _BigIntImpl _div(_BigIntImpl other) {
     assert(other._used > 0);
     if (_used < other._used) {
       return zero;
@@ -1921,8 +1927,7 @@ class _BigIntImpl implements BigInt {
   }
 
   /// Returns `this - other * trunc(this / other)`, with `other != 0`.
-  _BigIntImpl _rem(BigInt bigInt) {
-    _BigIntImpl other = bigInt;
+  _BigIntImpl _rem(_BigIntImpl other) {
     assert(other._used > 0);
     if (_used < other._used) {
       return this;
@@ -2116,8 +2121,7 @@ class _BigIntImpl implements BigInt {
   /// (-seven).remainder(three); // => -1
   /// seven.remainder(-three);   // => 1
   /// ```
-  _BigIntImpl operator ~/(BigInt bigInt) {
-    _BigIntImpl other = bigInt;
+  _BigIntImpl operator ~/(covariant _BigIntImpl other) {
     if (other._used == 0) {
       throw const IntegerDivisionByZeroException();
     }
@@ -2130,8 +2134,7 @@ class _BigIntImpl implements BigInt {
   /// `this == (this ~/ other) * other + r`.
   /// As a consequence the remainder `r` has the same sign as the divider
   /// `this`.
-  _BigIntImpl remainder(BigInt bigInt) {
-    _BigIntImpl other = bigInt;
+  _BigIntImpl remainder(covariant _BigIntImpl other) {
     if (other._used == 0) {
       throw const IntegerDivisionByZeroException();
     }
@@ -2142,16 +2145,16 @@ class _BigIntImpl implements BigInt {
   double operator /(BigInt other) => this.toDouble() / other.toDouble();
 
   /// Relational less than operator.
-  bool operator <(BigInt other) => compareTo(other) < 0;
+  bool operator <(covariant _BigIntImpl other) => compareTo(other) < 0;
 
   /// Relational less than or equal operator.
-  bool operator <=(BigInt other) => compareTo(other) <= 0;
+  bool operator <=(covariant _BigIntImpl other) => compareTo(other) <= 0;
 
   /// Relational greater than operator.
-  bool operator >(BigInt other) => compareTo(other) > 0;
+  bool operator >(covariant _BigIntImpl other) => compareTo(other) > 0;
 
   /// Relational greater than or equal operator.
-  bool operator >=(BigInt other) => compareTo(other) >= 0;
+  bool operator >=(covariant _BigIntImpl other) => compareTo(other) >= 0;
 
   /// Euclidean modulo operator.
   ///
@@ -2162,8 +2165,7 @@ class _BigIntImpl implements BigInt {
   /// The sign of the returned value `r` is always positive.
   ///
   /// See [remainder] for the remainder of the truncating division.
-  _BigIntImpl operator %(BigInt bigInt) {
-    _BigIntImpl other = bigInt;
+  _BigIntImpl operator %(covariant _BigIntImpl other) {
     if (other._used == 0) {
       throw const IntegerDivisionByZeroException();
     }
@@ -2222,9 +2224,8 @@ class _BigIntImpl implements BigInt {
   ///
   /// The [exponent] must be non-negative and [modulus] must be
   /// positive.
-  _BigIntImpl modPow(BigInt bigExponent, BigInt bigModulus) {
-    _BigIntImpl exponent = bigExponent;
-    _BigIntImpl modulus = bigModulus;
+  _BigIntImpl modPow(
+      covariant _BigIntImpl exponent, covariant _BigIntImpl modulus) {
     if (exponent._isNegative) {
       throw new ArgumentError("exponent must be positive: $exponent");
     }
@@ -2248,7 +2249,7 @@ class _BigIntImpl implements BigInt {
       resultDigits[j] = gDigits[j];
     }
     var resultUsed = gUsed;
-    var result2Used;
+    int result2Used;
     for (int i = exponentBitlen - 2; i >= 0; i--) {
       result2Used = z.sqr(resultDigits, resultUsed, result2Digits);
       if (!(exponent & (one << i))._isZero) {
@@ -2323,19 +2324,19 @@ class _BigIntImpl implements BigInt {
     // Variables a, b, c, and d require one more digit.
     final abcdUsed = maxUsed + 1;
     final abcdLen = abcdUsed + 2; // +2 to satisfy _absAdd.
-    var aDigits, bDigits, cDigits, dDigits;
-    bool aIsNegative, bIsNegative, cIsNegative, dIsNegative;
+    var aDigits = _dummyList;
+    var aIsNegative = false;
+    var cDigits = _dummyList;
+    var cIsNegative = false;
     if (ac) {
       aDigits = new Uint16List(abcdLen);
-      aIsNegative = false;
       aDigits[0] = 1;
       cDigits = new Uint16List(abcdLen);
-      cIsNegative = false;
     }
-    bDigits = new Uint16List(abcdLen);
-    bIsNegative = false;
-    dDigits = new Uint16List(abcdLen);
-    dIsNegative = false;
+    var bDigits = new Uint16List(abcdLen);
+    var bIsNegative = false;
+    var dDigits = new Uint16List(abcdLen);
+    var dIsNegative = false;
     dDigits[0] = 1;
 
     while (true) {
@@ -2511,7 +2512,7 @@ class _BigIntImpl implements BigInt {
       dIsNegative = false;
     } else {
       while ((dDigits[maxUsed] != 0) ||
-        (_compareDigits(dDigits, maxUsed, xDigits, maxUsed) >= 0)) {
+          (_compareDigits(dDigits, maxUsed, xDigits, maxUsed) >= 0)) {
         // d -= x
         _absSub(dDigits, abcdUsed, xDigits, maxUsed, dDigits);
       }
@@ -2526,8 +2527,7 @@ class _BigIntImpl implements BigInt {
   ///
   /// It is an error if no modular inverse exists.
   // Returns 1/this % modulus, with modulus > 0.
-  _BigIntImpl modInverse(BigInt bigInt) {
-    _BigIntImpl modulus = bigInt;
+  _BigIntImpl modInverse(covariant _BigIntImpl modulus) {
     if (modulus <= zero) {
       throw new ArgumentError("Modulus must be strictly positive: $modulus");
     }
@@ -2550,8 +2550,7 @@ class _BigIntImpl implements BigInt {
   /// For any integer `x`, `x.gcd(x)` is `x.abs()`.
   ///
   /// If both `this` and `other` is zero, the result is also zero.
-  _BigIntImpl gcd(BigInt bigInt) {
-    _BigIntImpl other = bigInt;
+  _BigIntImpl gcd(covariant _BigIntImpl other) {
     if (_isZero) return other.abs();
     if (other._isZero) return this.abs();
     return _binaryGcd(this, other, false);
@@ -2861,8 +2860,8 @@ class _BigIntClassic implements _BigIntReduction {
                 _modulus._digits[_modulus._used - 1].bitLength);
 
   int convert(_BigIntImpl x, Uint16List resultDigits) {
-    var digits;
-    var used;
+    Uint16List digits;
+    int used;
     if (x._isNegative || x._absCompare(_modulus) >= 0) {
       var remainder = x._rem(_modulus);
       if (x._isNegative && remainder._used > 0) {
